package org.fictus

trait Assertions {
    implicit def anyToAssertion[A](value: A): Assertion[A] = Assertion(value)

    def fail(): Nothing = fail("")
    def fail(msg: String): Nothing = throw new FailedError(msg)
}

case class Assertion[A](actual: A) {

    def mustMatch[B](success: PartialFunction[A, B]) = {
        if(success.isDefinedAt(actual)) success(actual)
        else throw new MatcherError("Unexpected value:\n" + actual + "\ndid not match your case!")
        this
    }

    def mustBe[B <: AnyRef](expected: B): Assertion[A] = actual match {
        case good: AnyRef if (expected eq good) => this
        case _                                  => throw new AssertionError("Expected\n" + expected + " but was\n" + actual)
    }

    def mustEqual[B <: A](expected: B): Assertion[A] = {
        (actual == expected) match {
            case true  => this
            case false => throw new AssertionError("Expected\n" + expected + " but was\n" + actual)
        }
    }

    def mustContain(expected: String): Assertion[A] = actual match {
        case s: String if (s.contains(expected)) => this
        case actual                              => throw new AssertionError("Expected " + actual + " to contain " + expected)
    }

    def mustStartWith(expected: String): Assertion[A] = actual match {
        case s: String if (s.startsWith(expected)) => this
        case actual                                => throw new AssertionError("Expected " + actual + " to start with " + expected)
    }

    def mustEndWith(expected: String): Assertion[A] = actual match {
        case s: String if (s.endsWith(expected)) => this
        case actual                              => throw new AssertionError("Expected " + actual + " to end with " + expected)
    }

    def mustEqualIgnoringCase(expected: String): Assertion[A] = actual match {
        case s: String if (s.equalsIgnoreCase(expected)) => this
        case unexpected                                  => throw new AssertionError("Expected\n" + expected + " but was\n" + unexpected)
    }

    override def equals(arg: Any) = throw new PebkacError("Using equals will lead to a buggy test... use equalTo instead")
}